
#include "main.h"
#include "uart0.h"

#define SHT1X_DDR			DDRD
#define SHT1X_PORT			PORTD
#define SHT1X_PIN			PIND
#define SHT1X_SCL			(1<<PD7)
#define SHT1X_SDA			(1<<PD4)

#define TRIGGER(x)	PORTB |= (1<<PB5); _delay_us((x)); PORTB &= ~(1<<PB5);

#define EMULATE

// http://de.wikipedia.org/wiki/Luftfeuchtigkeit
// http://de.wikipedia.org/wiki/Taupunkt

int main (void)
{
	char buffer[64];
	
	wdt_disable();
	ACSR	= (1<<ACD);							// Analog comparator disable
	MCUSR = 0;
	
	PORTB = 0;
	DDRB  = (1<<PB7);	// LED on crumb128 low-active
	DDRB  |= (1<<PB5);	// Trigger
	

	init_uart0( (u16)(F_CPU / BAUD / 16 - 0.5) );
	sei();
	_delay_ms( 1 );
	
	uputs0( "ASH500 Logger\n");
	
	
	
	
	
	
	// power-up ASH500 after sniffer
	// catch glitch
	while( !(SHT1X_PIN&SHT1X_SDA) );
	_delay_us(50);
	
	// soft reset
	
	// for some reason
	while( !(SHT1X_PIN&SHT1X_SCL) );	// until scl_hi
	while(  (SHT1X_PIN&SHT1X_SCL) );	// until scl_lo
	
	// 20 µs..
	
	// To initiate a transmission, a Transmission Start sequence has to be issued. 
	// It consists of a lowering of the DATA line while SCK is high, followed by a low pulse on SCK and raising DATA again while SCK is still high
	while( !(SHT1X_PIN&SHT1X_SCL) );	// until scl_hi
	while(  (SHT1X_PIN&SHT1X_SDA) );	// until sda_lo
	while(  (SHT1X_PIN&SHT1X_SCL) );	// until scl_lo
	while( !(SHT1X_PIN&SHT1X_SCL) );	// until scl_hi
	while( !(SHT1X_PIN&SHT1X_SDA) );	// until sda_hi
	while(  (SHT1X_PIN&SHT1X_SCL) );	// until scl_lo
	
	//~ TRIGGER(6)

	// sda goes low again as first 3 bits are zero
	
	// The subsequent command consists of three address bits (only '000' is supported) and five command bits.
	uint8_t data = 0;
	for( uint8_t i = 8; i; i-- ) {
		while( !(SHT1X_PIN&SHT1X_SCL) );		// until scl_hi
		if( SHT1X_PIN&SHT1X_SDA )
			data |= 1<< (i-1);
		 //~ PORTB ^= (1<<PB5);
		while( (SHT1X_PIN&SHT1X_SCL) );		// until scl_lo
	}
	
	//~ TRIGGER(6)
	
	// The SHT1x indicates the proper reception of a command by pulling the DATA pin low (ACK bit) after the falling edge of the 8th SCK clock. 
	
	#ifdef EMULATE
	SHT1X_DDR |= (SHT1X_SDA);		// output LOW
	#endif

	while( !(SHT1X_PIN&SHT1X_SCL) );	// until scl_hi
	while( (SHT1X_PIN&SHT1X_SCL) );		// until scl_lo
	// The DATA line is released (and goes high) after the falling edge of the 9th SCK clock. 
	
	#ifdef EMULATE
	SHT1X_DDR &= ~(SHT1X_SDA);		// tristate
	#else
	while( !(SHT1X_PIN&SHT1X_SDA) );	// until sda_hi
	#endif
	
	//~ TRIGGER(6)
	
	// data should contain the command now
	uputchar0( data);
	//~ uputs0( "\n");

	// this was a soft reset 00011110 = 0x1e = 30


	// connection reset sequence
	// 16 clock high pulses while data high
	for( uint8_t i = 18; i; i-- ) {
		while( !(SHT1X_PIN&SHT1X_SCL) );	// until scl_hi
		while( (SHT1X_PIN&SHT1X_SCL) );		// until scl_lo
	}

	//~ TRIGGER(6)

	// read commands and two byte data
	uint8_t dat[2][3] =	{	{3, 6, 0x78},	// temperature
							{5, 0, 0x40}	// rel. humidity
						};

	

	while(1) {
		
		// temp and humidity
		for( uint8_t n = 0; n<2; n++ ) {
			
			//~ if( n == 1 ) {
					//~ TRIGGER(6)
				//~ }
			
			// for some reason
			while( !(SHT1X_PIN&SHT1X_SCL) );	// until scl_hi
			while(  (SHT1X_PIN&SHT1X_SCL) );	// until scl_lo
			
			// To initiate a transmission, a Transmission Start sequence has to be issued. 
			// It consists of a lowering of the DATA line while SCK is high, followed by a low pulse on SCK and raising DATA again while SCK is still high
			while( !(SHT1X_PIN&SHT1X_SCL) );	// until scl_hi
			while(  (SHT1X_PIN&SHT1X_SDA) );	// until sda_lo
			while(  (SHT1X_PIN&SHT1X_SCL) );	// until scl_lo
			while( !(SHT1X_PIN&SHT1X_SCL) );	// until scl_hi
			while( !(SHT1X_PIN&SHT1X_SDA) );	// until sda_hi
			while(  (SHT1X_PIN&SHT1X_SCL) );	// until scl_lo
			
			
			//~ TRIGGER(6)
			
			// command and two bytes answer
			// Two bytes of measurement data and one byte of CRC checksum (optional) will then be transmitted. 
			// The microcontroller must acknowledge each byte by pulling the DATA line low. 
			// All values are MSB first, right justified (e.g. the 5 th SCK is MSB for a 12bit value, for a 8bit result the first byte is not used). 
			for( uint8_t bytes = 0; bytes < 3; bytes++) {
				
				if( n == 0 && bytes == 2) {
					TRIGGER(6)
				}
				
				// The subsequent command consists of three address bits (only '000' is supported) and five command bits.
				uint8_t data = 0;
				
				#ifdef EMULATE
				if( bytes > 0) {
					data = dat[n][bytes];
				}
				#endif
				
				for( uint8_t i = 8; i; i-- ) {
					while( !(SHT1X_PIN&SHT1X_SCL) );	// until scl_hi
					
					#ifdef EMULATE
					if( bytes == 0)
					#endif
					if( SHT1X_PIN&SHT1X_SDA )
						data |= 1<< (i-1);
					
					
					#ifdef EMULATE
					if( bytes > 0) {
						if( ~data & (1<<i-1) )
							SHT1X_DDR |= (SHT1X_SDA);		// output LOW
						else
							SHT1X_DDR &= ~(SHT1X_SDA);		// tristate
					}
					#endif
					
					while( (SHT1X_PIN&SHT1X_SCL) );		// until scl_lo
					
					#ifdef EMULATE
					if( bytes > 0) {
						if( data & (1<<i-2))
							SHT1X_DDR &= ~(SHT1X_SDA);		// tristate
						else
							SHT1X_DDR |= (SHT1X_SDA);		// output LOW
					}
					#endif
				}
				// The SHT1x indicates the proper reception of a command by pulling the DATA pin low (ACK bit) after the falling edge of the 8th SCK clock. 
				
				#ifdef EMULATE
				if( bytes == 0)
					SHT1X_DDR |= (SHT1X_SDA);		// output LOW
				else
					SHT1X_DDR &= ~(SHT1X_SDA);		// tristate
				#endif

				// The DATA line is released (and goes high) after the falling edge of the 9th SCK clock. 
				while( !(SHT1X_PIN&SHT1X_SCL) );	// until scl_hi
				while( (SHT1X_PIN&SHT1X_SCL) );		// until scl_lo
				#ifdef EMULATE
				if( bytes == 0)
					SHT1X_DDR &= ~(SHT1X_SDA);		// tristate
				#endif
				
				//~ TRIGGER(6)
				
				//~ uputchar0( data);
				dat[n][bytes] = data;
				
				
				// After issuing a measurement command ('00000101' for relative  humidity, '00000011' for temperature) the controller has to wait for the measurement to complete. 
				// To signal the completion of a measurement, the SHT1x pulls data line low and enters Idle Mode. 
				
				#ifdef EMULATE
				if( bytes == 0) {
					_delay_ms(40);
					SHT1X_DDR |= (SHT1X_SDA);		// output LOW
				}
				#endif
				
				
				
				
				
				//~ while(  (SHT1X_PIN&SHT1X_SDA) );	// until sda_lo
				//~ TRIGGER(6)
				
				PORTB  ^= (1<<PB7);
			}
		}
		
		
		// output
		for( uint8_t n = 0; n<2; n++ ) {
			for( uint8_t bytes = 0; bytes < 3; bytes++) {
				sprintf( buffer, "0x%0x ", dat[n][bytes]);
				uputs0( buffer );
			}
		}
		
		
		// get ASH500 TX data
		// pulse width about 200..300 µs
		
		// detect start of transmission
		while( !(PINF & (1<<PF0)) );	// rising edge
		
		
		uputchar0( '0' );	// zero level
		//~ PORTB |= (1<<PB5);	// TRIGGER on
		
		//~ TRIGGER(6)
		
		// start timer
		TCNT0 = 0;
		TCCR0A = (1<<CS01) | (1<<CS00);	// clk_IO / 64
		
		uint8_t state = (1<<PF0);		// save state
		uint8_t time = 0;
		
		// get the whole transmission
		while(1) {
			
			// edge or timeout
			while( !( (PINF & (1<<PF0)) ^ state) && TCNT0 <255);	// 
			
			//~ PORTB ^= (1<<PB5);	// TRIGGER
			//~ TRIGGER(6)
			
			time = TCNT0;
			TCNT0 = 0;
			
			// timeout -> done
			if( time > 254) 
				break;
			
			// detect pulse-width
			if( time > 20 && time < 40 ) {
				// short
				
				//~ PORTB |= (1<<PB5);	// TRIGGER
				//~ PORTB ^= (1<<PB5);	// TRIGGER
				if( state )
					uputchar0('1');
				else
					uputchar0('0');
					
			} else
			if( time >= 55 && time < 65 ) {
				// long
				//~ TRIGGER(6)
				//~ PORTB &= ~(1<<PB5);	// TRIGGER
				//~ PORTB ^= (1<<PB5);	// TRIGGER
				
				if( state )
					uputs0("11");
				else
					uputs0("00");
			}
			
			//~ uputchar0( time );
			
			// save state
			state = PINF & (1<<PF0);
		}
		
		//~ TRIGGER(15)
		
		// done
		uputchar0( '\n' );
		
		// 21 * every 9 seconds after power-up
		// every 3 min after
	}
}



